@@ -1349,23 +1349,29 @@ static const GladeKeyDef keys[] = {
   { NULL, 0 }
 };
 
-static GHashTable *key_hash() {
-  static GHashTable *hash = NULL;
-  if (!hash) {
-    int i = 0;
-    hash = g_hash_table_new(g_str_hash, g_str_equal);
-    g_hash_table_freeze(hash);
-    while (keys[i].str != NULL) {
-      g_hash_table_insert(hash, keys[i].str, GUINT_TO_POINTER(keys[i].key));
-      i++;
-    }
-    g_hash_table_thaw(hash);
-  }
-  return hash;
+static GHashTable *
+key_hash()
+{
+	static GHashTable *hash = NULL;
+	if (!hash) {
+		int i = 0;
+		hash = g_hash_table_new(g_str_hash, g_str_equal);
+		g_hash_table_freeze(hash);
+		while (keys[i].str != NULL) {
+			g_hash_table_insert(hash, keys[i].str, GUINT_TO_POINTER(keys[i].key));
+			i++;
+		}
+		g_hash_table_thaw(hash);
+	}
+	return hash;
 }
 
-guint glade_key_get(char *str) {
-  guint ret = GPOINTER_TO_UINT(g_hash_table_lookup(key_hash(), str));
-  g_return_val_if_fail(ret != 0, 0);
-  return ret;
+guint
+glade_key_get(char *str)
+{
+	guint ret = GPOINTER_TO_UINT(g_hash_table_lookup(key_hash(), str));
+	g_return_val_if_fail(ret != 0, 0);
+	return ret;
 }
+
+
