@@ -25,9 +25,9 @@
 #include "sheet-object.h"
 #include "style.h"
 
-#include "ms-ole.h"
+#include "excel.h"
 #include "ms-biff.h"
-#include "ms-excel.h"
+#include "ms-excel-read.h"
 #include "ms-excel-biff.h"
 #include "ms-obj.h"
 
