@@ -13,8 +13,11 @@
 #include "format.h"
 #include "color.h"
 #include "cursors.h"
+#include "utils.h"
+#include <ctype.h>
 
-static int         redraws_frozen = 0;
+static int         redraws_frozen      = 0;
+static int         redraws_deep_frozen = 0;
 static GHashTable *cell_hash_queue;
 
 void
@@ -36,11 +39,12 @@ cell_modified (Cell *cell)
 		sheet->modified = TRUE;
 }
 
+
 void
 cell_set_formula (Cell *cell, const char *text)
 {
 	char *error_msg = _("ERROR");
-	char *desired_format = NULL;
+	const char *desired_format = NULL;
 	
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (text != NULL);
@@ -64,7 +68,7 @@ cell_set_formula (Cell *cell, const char *text)
 		if (cell->flags & CELL_ERROR)
 			cell->flags &= ~CELL_ERROR;
 	}
-	
+
 	if (desired_format && strcmp (cell->style->format->format, "General") == 0){
 		style_format_unref (cell->style->format);
 		cell->style->format = style_format_new (desired_format);
@@ -346,7 +350,7 @@ cell_unrealize (Cell *cell)
 }
 
 void
-cell_set_comment (Cell *cell, char *str)
+cell_set_comment (Cell *cell, const char *str)
 {
 	int had_comments = FALSE;
 	
@@ -535,6 +539,66 @@ cell_render_value (Cell *cell)
 	g_free (str);
 }
 
+/*
+ * Sets the value for a cell:
+ *
+ * This is kind of an internal function and should be only called by
+ * routines that know what they are doing.  These are the important
+ * differences from cell_set_value:
+ *
+ *    - It does not queue redraws (so you have to queue the redraw yourself
+ *      or queue a full redraw).
+ *
+ *    - It does not queue any recomputations.  You have to queue the recompute
+ *      yourself.
+ */
+void
+cell_set_value_simple (Cell *cell, Value *v)
+{
+	struct lconv *lconv;
+	
+	g_return_if_fail (cell);
+	g_return_if_fail (v);
+
+	cell_modified (cell);
+
+	if (cell->entered_text)
+		string_unref (cell->entered_text);
+	cell->entered_text = NULL;
+					 
+	if (cell->value)
+		value_release (cell->value);
+	
+	if (cell->parsed_node){
+		sheet_cell_formula_unlink (cell);
+
+		expr_tree_unref (cell->parsed_node);
+		cell->parsed_node = NULL;
+	}
+
+	cell->value = v;
+	cell_render_value (cell);
+}
+
+/*
+ * cell_set_value
+ *
+ * Changes the value of a cell
+ */
+void
+cell_set_value (Cell *cell, Value *v)
+{
+	g_return_if_fail (cell);
+	g_return_if_fail (v);
+	
+	cell_queue_redraw (cell);
+
+	cell_set_value_simple (cell, v);
+	cell_content_changed (cell);
+
+	cell_queue_redraw (cell);
+}
+
 /*
  * Sets the text for a cell:
  *
@@ -577,73 +641,32 @@ cell_set_text_simple (Cell *cell, const char *text)
  	if (text [0] == '=' && text [1] != 0){
 		cell_set_formula (cell, text); 
 	} else {
-		Value *v = g_new (Value, 1);
-		int is_text, is_float, maybe_float, has_digits;
-		int seen_exp;
 		const char *p;
+		char *end;
+		long l;
 
-		lconv = localeconv ();
-		
-		is_text = is_float = maybe_float = has_digits = FALSE;
-		seen_exp = FALSE;
-		for (p = text; *p && !is_text; p++){
-			switch (*p){
-			case '0': case '1': case '2': case '3': case '4':
-			case '5': case '6': case '7': case '8': case '9':
-				has_digits = TRUE;
-				break;
+		/* Skip spaces, just in case.  */
+		p = text;
+		while (isspace (*p)) p++;
 
-			case '-':
-				if (p == text)
-					break;
-				if (seen_exp)
-					is_text = TRUE;
-				/* falldown */
-				
-			case 'E': case 'e': case '+': case ':': case '.': case ',':
-				if (*p == 'e' || *p == 'E') {
-					seen_exp = TRUE;
-
-					/* Lookahead */
-					if (*(p+1) == '-')
-						p++; 
-				}
-				
-				if (*p == ',' || *p == '.')
-					if (*lconv->decimal_point != *p){
-						is_text = TRUE;
-						break;
-					}
-				maybe_float = TRUE;
-				break;
-
-			default:
-				is_text = TRUE;
-			}
-		}
-		if (has_digits && maybe_float)
-			is_float = TRUE;
-		
-		if (has_digits && !is_text){
-			if (is_float){
-				v->type = VALUE_FLOAT;
-				float_get_from_range (text, text+strlen(text),
-						      &v->v.v_float);
-			} else {
-				v->type = VALUE_INTEGER;
-				int_get_from_range (text, text+strlen (text),
-						    &v->v.v_int);
-			}
+		l = strtol (p, &end, 10);
+		if (p != end && *end == 0) {
+			/* It is an int.  FIXME: long/int confusion here.  */
+			cell->value = value_new_int (l);
 		} else {
-			v->type = VALUE_STRING;
-			v->v.str = string_get (text);
+			double d;
+			d = strtod (p, &end);
+			if (p != end && *end == 0) {
+				/* It is a floating point number.  */
+				cell->value = value_new_float ((float_t)d);
+			} else {
+				/* It is text.  */
+				cell->value = value_new_string (text);
+			}
 		}
-		cell->value = v;
-		
-		cell_render_value (cell);
-	}
 
-	
+		cell_render_value (cell);
+	}	
 }
 
 /*
@@ -754,13 +777,23 @@ cell_copy (const Cell *cell)
 	new_cell->col   = NULL;
 	new_cell->row   = NULL;
 	new_cell->sheet = NULL;
-	
-	/* now copy propertly the rest */
+	new_cell->flags &= ~CELL_QUEUED_FOR_RECALC;
+
+	/* now copy properly the rest */
 	if (new_cell->parsed_node)
-		expr_tree_ref   (new_cell->parsed_node);
-	string_ref      (new_cell->text);
-	
-	new_cell->style = style_duplicate (new_cell->style);
+		expr_tree_ref (new_cell->parsed_node);
+
+	if (new_cell->text)
+		string_ref (new_cell->text);
+
+	if (new_cell->entered_text)
+		string_ref (new_cell->entered_text);
+
+	if (new_cell->style)
+		new_cell->style = style_duplicate (new_cell->style);
+
+	if (new_cell->render_color)
+		style_color_ref (new_cell->render_color);
 
 	/*
 	 * The cell->value can be NULL if the cell contains
@@ -769,9 +802,10 @@ cell_copy (const Cell *cell)
 	if (new_cell->value)
 		new_cell->value = value_duplicate (new_cell->value);
 
-	new_cell->comment = NULL;
-	if (cell->comment)
+	if (cell->comment) {
+		new_cell->comment = NULL;
 		cell_set_comment (new_cell, cell->comment->comment->str);
+	}
 
 	return new_cell;
 }
@@ -781,24 +815,38 @@ cell_destroy (Cell *cell)
 {
 	g_return_if_fail (cell != NULL);
 
+	if (cell_hash_queue && g_hash_table_lookup (cell_hash_queue, cell)) {
+		g_warning ("FIXME: Deleting cell %s which was queued for redraw",
+			   cell_name (cell->col->pos, cell->row->pos));
+		g_hash_table_remove (cell_hash_queue, cell);
+	}
+
 	cell_modified (cell);
 
-	if (cell->parsed_node){
+	if (cell->parsed_node)
 		expr_tree_unref (cell->parsed_node);
-	}
+	cell->parsed_node = (void *)0xdeadbeef;
 
 	if (cell->render_color)
 		style_color_unref (cell->render_color);
+	cell->render_color = (void *)0xdeadbeef;
 
 	cell_comment_destroy (cell);
 
 	if (cell->text)
-		string_unref  (cell->text);
+		string_unref (cell->text);
+	cell->text = (void *)0xdeadbeef;
+
+	if (cell->entered_text)
+		string_unref (cell->entered_text);
+	cell->entered_text = (void *)0xdeadbeef;
 
 	style_destroy (cell->style);
+	cell->style = (void *)0xdeadbeef;
 
 	if (cell->value)
 		value_release (cell->value);
+	cell->value = (void *)0xdeadbeef;
 
 	g_free (cell);
 }
@@ -831,6 +879,19 @@ cell_thaw_redraws (void)
 		cell_hash_queue = NULL;
 	}
 }
+void
+cell_deep_freeze_redraws (void)
+{
+	redraws_deep_frozen++;
+}
+
+void
+cell_deep_thaw_redraws (void)
+{
+	redraws_deep_frozen--;
+	if (redraws_frozen < 0)
+		g_warning ("unbalanced deep freeze/thaw\n");
+}
 
 static void
 queue_cell (Cell *cell)
@@ -843,13 +904,17 @@ queue_cell (Cell *cell)
 void
 cell_queue_redraw (Cell *cell)
 {
+	/* You wake up dead after a deep freeze */
+	if (redraws_deep_frozen>0)
+		return;
+
 	g_return_if_fail (cell != NULL);
 
 	if (redraws_frozen){
 		queue_cell (cell);
 		return;
 	}
-	
+
 	sheet_redraw_cell_region (cell->sheet,
 				  cell->col->pos, cell->row->pos,
 				  cell->col->pos, cell->row->pos);
@@ -955,15 +1020,18 @@ cell_relocate (Cell *cell, int col_diff, int row_diff)
 
 	/* 2. If the cell contains a formula, relocate the formula */
 	if (cell->parsed_node){
+		sheet_cell_formula_unlink (cell);
 		if (col_diff != 0 || row_diff != 0){
 			ExprTree *new_tree;
-			char *text, *formula;
-			
+
 			new_tree = expr_tree_relocate (cell->parsed_node, col_diff, row_diff);
 
-			expr_tree_unref (cell->parsed_node);
-			cell->parsed_node = new_tree;
+			if (new_tree) {
+				expr_tree_unref (cell->parsed_node);
+				cell->parsed_node = new_tree;
+			}
 		}
+		/* The following call also relinks the cell.  */
 		cell_formula_changed (cell);
 	}
 	
@@ -1402,11 +1470,10 @@ str_trim_spaces (char *s)
  * Returns the number of columns used for the draw
  */
 int 
-cell_draw (Cell *cell, void *sv, GdkGC *gc, GdkDrawable *drawable, int x1, int y1)
+cell_draw (Cell *cell, SheetView *sheet_view, GdkGC *gc, GdkDrawable *drawable, int x1, int y1)
 {
 	Style        *style = cell->style;
 	GdkFont      *font = style->font->font;
-	SheetView    *sheet_view = sv;
 	GdkGC        *white_gc = GTK_WIDGET (sheet_view->sheet_view)->style->white_gc;
 	GdkRectangle rect;
 	
@@ -1667,7 +1734,7 @@ cell_get_content (Cell *cell)
 	 * Return the value wihtout parsing.
 	 */
 	if (cell->value)
-		str = value_string (cell->value);
+		str = value_get_as_string (cell->value);
 	else
 		str = g_strdup (cell->entered_text->str);
 	
