@@ -145,8 +145,6 @@ add_cell_range_deps (Cell *cell, const CellRef *a, const CellRef *b)
 static void
 add_value_deps (Cell *cell, const Value *value)
 {
-	GList *l;
-	
 	switch (value->type){
 	case VALUE_STRING:
 	case VALUE_INTEGER:
@@ -162,7 +160,7 @@ add_value_deps (Cell *cell, const Value *value)
 		for (x = 0; x < value->v.array.x; x++)
 			for (y = 0; y < value->v.array.y; y++)
 				add_value_deps (cell,
-						&value->v.array.vals [x][y]);
+						value->v.array.vals [x][y]);
 		break;
 	}
 	case VALUE_CELLRANGE:
@@ -184,22 +182,15 @@ add_tree_deps (Cell *cell, ExprTree *tree)
 	GList *l;
 	
 	switch (tree->oper){
-	case OPER_EQUAL:
-	case OPER_NOT_EQUAL:
-	case OPER_GT:
-	case OPER_GTE:
-	case OPER_LT:
-	case OPER_LTE:
-	case OPER_ADD:
-	case OPER_SUB:
-	case OPER_MULT:
-	case OPER_DIV:
-	case OPER_EXP:
-	case OPER_CONCAT:
+	case OPER_ANY_BINARY:
 		add_tree_deps (cell, tree->u.binary.value_a);
 		add_tree_deps (cell, tree->u.binary.value_b);
 		return;
 
+	case OPER_ANY_UNARY:
+		add_tree_deps (cell, tree->u.value);
+		return;
+	
 	case OPER_VAR: 
 		add_cell_range_deps (
 			cell,
@@ -216,10 +207,6 @@ add_tree_deps (Cell *cell, ExprTree *tree)
 			add_tree_deps (cell, l->data);
 		return;
 
-	case OPER_NEG:
-		add_tree_deps (cell, tree->u.value);
-		return;
-	
 	} /* switch */
 }
 
@@ -233,7 +220,7 @@ cell_add_dependencies (Cell *cell)
 {
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (cell->parsed_node != NULL);
-	
+
 	if (!dependency_hash)
 		dependency_hash_init ();
 
@@ -252,14 +239,15 @@ static void
 dependency_remove_cell (gpointer key, gpointer value, gpointer the_cell)
 {
 	DependencyRange *range = value;
-	GList *list = range->cell_list;
+	GList *list;
 
 	list = g_list_find (range->cell_list, the_cell);
 	if (!list)
 		return;
 
 	range->cell_list = g_list_remove_link (range->cell_list, list);
-	
+	g_list_free_1 (list);
+
 	range->ref_count--;
 
 	if (range->ref_count == 0)
@@ -304,7 +292,8 @@ typedef struct {
 static gboolean
 intersects (Sheet *sheet, int col, int row, DependencyRange *range)
 {
-	if (range_contains (&range->range, col, row) && (sheet == range->sheet))
+	if ((sheet == range->sheet) &&
+	    range_contains (&range->range, col, row))
 		return TRUE;
 
 	return FALSE;
@@ -317,13 +306,17 @@ search_cell_deps (gpointer key, gpointer value, gpointer closure)
 	get_dep_closure_t *c = closure;
 	Sheet *sheet = c->sheet;
 	GList *l;
+	static int inters=0;
+	static int nointers=0;
+
+	/* No intersection is the common case */
 
 	if (!(intersects (sheet, c->start_col, c->start_row, range) ||
 	      intersects (sheet, c->end_col,   c->end_row,   range) ||
 	      intersects (sheet, c->start_col, c->end_row,   range) ||
 	      intersects (sheet, c->end_col,   c->start_row, range)))
 		return;
-			
+
 	for (l = range->cell_list; l; l = l->next){
 		Cell *cell = l->data;
 
@@ -384,6 +377,9 @@ cell_queue_recalc (Cell *cell)
 
 	g_return_if_fail (cell != NULL);
 	
+	if (cell->flags & CELL_QUEUED_FOR_RECALC)
+		return;
+
 	wb = ((Sheet *)cell->sheet)->workbook;
 	wb->eval_queue = g_list_prepend (wb->eval_queue, cell);
 	cell->flags |= CELL_QUEUED_FOR_RECALC;
@@ -417,14 +413,27 @@ cell_queue_recalc_list (GList *list)
 {
 	Workbook *wb;
 	Cell *first_cell;
-	
+	GList *list0 = list;
+
 	if (!list)
 		return;
 
 	first_cell = list->data;
 	wb = ((Sheet *)(first_cell->sheet))->workbook;
 
-	wb->eval_queue = g_list_concat (wb->eval_queue, list);
+	while (list) {
+		Cell *cell = list->data;
+		list = list->next;
+
+		if (cell->flags & CELL_QUEUED_FOR_RECALC)
+			continue;
+
+		wb->eval_queue = g_list_prepend (wb->eval_queue, cell);
+
+		cell->flags |= CELL_QUEUED_FOR_RECALC;
+	}
+
+	g_list_free (list0);
 }
 
 static Cell *
@@ -437,6 +446,8 @@ pick_next_cell_from_queue (Workbook *wb)
 	
 	cell = wb->eval_queue->data;
 	wb->eval_queue = g_list_remove (wb->eval_queue, cell);
+	if (!(cell->flags & CELL_QUEUED_FOR_RECALC))
+		printf ("De-queued cell here\n");
 	cell->flags &= ~CELL_QUEUED_FOR_RECALC;
 	return cell;
 }
