@@ -69,6 +69,7 @@ file_format_unregister_open (FileFormatProbe probe, FileFormatOpen open)
 
 		if (fo->probe == probe && fo->open == open){
 			gnumeric_file_openers = g_list_remove_link (gnumeric_file_openers, l);
+			g_list_free_1 (l);
 			return;
 		}
 	}
@@ -93,7 +94,7 @@ file_format_register_save (char *extension, char *format_description, FileFormat
 	fs->format_description = format_description;
 	fs->save = save_fn;
 
-	gnumeric_file_savers = g_list_prepend (gnumeric_file_savers, fs);
+	gnumeric_file_savers = g_list_append (gnumeric_file_savers, fs);
 }
 
 /**
@@ -115,6 +116,7 @@ file_format_unregister_save (FileFormatSave save)
 				current_saver = NULL;
 			
 			gnumeric_file_savers = g_list_remove_link (gnumeric_file_savers, l);
+			g_list_free_1 (l);
 			return;
 		}
 	}
