@@ -34,6 +34,7 @@
 #include "gnumeric.h"
 #include "format.h"
 #include "dates.h"
+#include "utils.h"
 
 /* Points to the locale information for number display */
 static struct lconv *lc;
@@ -660,6 +661,8 @@ split_time (gdouble number)
 	secs -= tm.tm_min * 60;
 	tm.tm_sec  = floor (secs);
 
+	g_date_free (date);
+
 	return &tm;
 }
 
@@ -1019,13 +1022,19 @@ format_value (StyleFormat *format, Value *value, StyleColor **color)
 	case VALUE_STRING:
 		return g_strdup (value->v.str->str);
 
+	case VALUE_CELLRANGE:
+		return g_strdup (_("CELLRANGE"));
+
+	case VALUE_ARRAY:
+		return g_strdup (_("ARRAY"));
+		
 	default:
 		return g_strdup ("Internal error");
 	}
 
 	/* Format error, return a default value */
 	if (v == NULL)
-		return value_string (value);
+		return value_get_as_string (value);
 
 	return v;
 }
