@@ -89,7 +89,7 @@ function_iterate_do_value (Sheet                   *sheet,
 				ret = function_iterate_do_value (
 					sheet, callback, closure,
 					eval_col, eval_row,
-					&value->v.array.vals [x][y], error_string);
+					value->v.array.vals [x][y], error_string);
 				if (ret == FALSE)
 					return FALSE;
 			}
@@ -270,6 +270,9 @@ functions_init (void)
 	install_symbols (finance_functions, _("Financial"));
 	install_symbols (eng_functions, _("Engineering"));
 	install_symbols (lookup_functions, _("Data / Lookup"));
+	install_symbols (logical_functions, _("Logical"));
+	install_symbols (database_functions, _("Database"));
+	install_symbols (information_functions, _("Information"));
 }
 
 void
@@ -295,4 +298,8 @@ constants_init (void)
 	symbol_install (global_symbol_table, "FALSE", SYMBOL_VALUE, false);
 	symbol_install (global_symbol_table, "TRUE", SYMBOL_VALUE, true);
 	symbol_install (global_symbol_table, "GNUMERIC_VERSION", SYMBOL_VALUE, version);
+
+	/* Global helper value for arrays */
+	value_zero = value_new_float (0);
 }
+
