@@ -1,6 +1,9 @@
 #ifndef GNUMERIC_FUNC_H
 #define GNUMERIC_FUNC_H
 
+#include "expr.h"
+#include "sheet.h"
+
 extern FunctionDefinition math_functions [];
 extern FunctionDefinition sheet_functions [];
 extern FunctionDefinition misc_functions [];
@@ -10,6 +13,9 @@ extern FunctionDefinition stat_functions [];
 extern FunctionDefinition finance_functions [];
 extern FunctionDefinition eng_functions [];
 extern FunctionDefinition lookup_functions [];
+extern FunctionDefinition logical_functions [];
+extern FunctionDefinition database_functions [];
+extern FunctionDefinition information_functions [];
 
 typedef int (*FunctionIterateCallback)(Sheet *sheet, Value *value, char **error_string, void *);
 
@@ -77,22 +83,54 @@ typedef struct {
 	FunctionDefinition *fd ;
 } TokenizedHelp;
 
+typedef struct {
+	int N;
+	float_t M, Q;
+} stat_closure_t;
+
 TokenizedHelp *tokenized_help_new     (FunctionDefinition *fd) ;
 char          *tokenized_help_find    (TokenizedHelp *tok, char *token) ;
 void           tokenized_help_destroy (TokenizedHelp *tok) ;
 
 float_t combin (int n, int k);
 float_t fact   (int n);
+void setup_stat_closure (stat_closure_t *cl);
+int callback_function_stat (Sheet *sheet, Value *value, char **error_string,
+			    void *closure);
 
-Value *gnumeric_average     (void *tsheet, GList *expr_node_list,
+Value *gnumeric_average     (Sheet *sheet, GList *expr_node_list,
 			     int eval_col, int eval_row,
 			     char **error_string);
 
-Value *gnumeric_count       (void *tsheet, GList *expr_node_list,
+Value *gnumeric_count       (Sheet *sheet, GList *expr_node_list,
 			     int eval_col, int eval_row,
 			     char **error_string);
 
+Value *gnumeric_sum         (Sheet *sheet, GList *expr_node_list,
+			     int eval_col, int eval_row,
+			     char **error_string);
+
+/* Type definitions and function prototypes for criteria functions.
+ * This includes the database functions and some mathematical functions
+ * like COUNTIF, SUMIF...
+ */
+typedef int (*criteria_test_fun_t) (Value *x, Value *y);
+
+typedef struct {
+        criteria_test_fun_t fun;
+        Value                *x;
+} func_criteria_t;
+
+int criteria_test_equal(Value *x, Value *y);
+int criteria_test_unequal(Value *x, Value *y);
+int criteria_test_greater(Value *x, Value *y);
+int criteria_test_less(Value *x, Value *y);
+int criteria_test_greater_or_equal(Value *x, Value *y);
+int criteria_test_less_or_equal(Value *x, Value *y);
+void parse_criteria(char *criteria, criteria_test_fun_t *fun,
+		    Value **test_value);
+
+
 int solver_simplex(Workbook *wb, Sheet *sheet);
 
 #endif /* GNUMERIC_FUNC_H */
-
