@@ -33,6 +33,10 @@
 #include <gnome.h>
 #include "gnumeric.h"
 #include "number-match.h"
+#include "symbol.h"
+#include "expr.h"
+#include "sheet.h"
+#include "utils.h"
 	
 /* Allocation with disposal-on-error */ 
 static void *alloc_buffer    (int size);
@@ -42,7 +46,7 @@ static void  alloc_glist     (GList *l);
 static void  forget_glist    (GList *list);
 static void  forget_tree     (ExprTree *tree);
 static void  alloc_list_free (void); 
-static void *v_new (void);
+static Value*v_new (void);
 	
 #define ERROR -1
  
@@ -70,21 +74,38 @@ static void dump_tree (ExprTree *tree);
 static int  yylex (void);
 static int  yyerror (char *s);
 
+/* The expression being parsed */
+static const char *parser_expr;
+
+/* The error returned from the */
+static ParseErr parser_error;
+
+/* The sheet where the parsing takes place */
+static Sheet *parser_sheet;
+
+/* Location where the parsing is taking place */
+static int parser_col, parser_row;
+
+/* The suggested format to use for this expression */
+static const char **parser_desired_format;
+
+static ExprTree **parser_result;
+
 #define p_new(type) ((type *) alloc_buffer ((unsigned) sizeof (type)))
 
 static ExprTree *
 build_binop (ExprTree *l, Operation op, ExprTree *r)
 {
-  ExprTree *res = p_new (ExprTree);
-  res->ref_count = 1;
-  res->u.binary.value_a = l;
-  res->oper = op;
-  res->u.binary.value_b = r;
-  return res;
+	ExprTree *res = p_new (ExprTree);
+	res->ref_count = 1;
+	res->u.binary.value_a = l;
+	res->oper = op;
+	res->u.binary.value_b = r;
+	return res;
 }
 
 
-#line 68 "parser.y"
+#line 89 "parser.y"
 typedef union {
 	ExprTree *tree;
 	CellRef  *cell;
@@ -161,9 +182,9 @@ static const short yyrhs[] = {    29,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-    89,    93,    99,   100,   101,   102,   103,   104,   105,   106,
-   107,   108,   109,   110,   111,   112,   113,   114,   115,   117,
-   124,   128,   147,   153,   157,   163,   167,   172
+   110,   111,   114,   115,   116,   117,   118,   119,   120,   121,
+   122,   123,   124,   125,   126,   127,   128,   129,   130,   132,
+   139,   143,   157,   163,   167,   173,   177,   182
 };
 #endif
 
@@ -749,89 +770,83 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 89 "parser.y"
-{ parser_result = yyvsp[0].tree;
-                          /* dump_tree (parser_result);*/
-                          alloc_list_free (); 
-                        ;
+#line 110 "parser.y"
+{ *parser_result = yyvsp[0].tree; ;
     break;}
 case 2:
-#line 93 "parser.y"
-{
-				alloc_clean ();
-				parser_error = PARSE_ERR_SYNTAX;
-			;
+#line 111 "parser.y"
+{ parser_error = PARSE_ERR_SYNTAX; ;
     break;}
 case 3:
-#line 99 "parser.y"
+#line 114 "parser.y"
 { yyval.tree = yyvsp[0].tree ;
     break;}
 case 4:
-#line 100 "parser.y"
+#line 115 "parser.y"
 { yyval.tree = yyvsp[0].tree ;
     break;}
 case 5:
-#line 101 "parser.y"
+#line 116 "parser.y"
 { yyval.tree = yyvsp[0].tree ;
     break;}
 case 6:
-#line 102 "parser.y"
+#line 117 "parser.y"
 { yyval.tree = yyvsp[0].tree ;
     break;}
 case 7:
-#line 103 "parser.y"
+#line 118 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_ADD,       yyvsp[0].tree); ;
     break;}
 case 8:
-#line 104 "parser.y"
+#line 119 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_SUB,       yyvsp[0].tree); ;
     break;}
 case 9:
-#line 105 "parser.y"
+#line 120 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_MULT,      yyvsp[0].tree); ;
     break;}
 case 10:
-#line 106 "parser.y"
+#line 121 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_DIV,       yyvsp[0].tree); ;
     break;}
 case 11:
-#line 107 "parser.y"
+#line 122 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_EXP,       yyvsp[0].tree); ;
     break;}
 case 12:
-#line 108 "parser.y"
+#line 123 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_CONCAT,    yyvsp[0].tree); ;
     break;}
 case 13:
-#line 109 "parser.y"
+#line 124 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_EQUAL,     yyvsp[0].tree); ;
     break;}
 case 14:
-#line 110 "parser.y"
+#line 125 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_LT,        yyvsp[0].tree); ;
     break;}
 case 15:
-#line 111 "parser.y"
+#line 126 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_GT,        yyvsp[0].tree); ;
     break;}
 case 16:
-#line 112 "parser.y"
+#line 127 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_GTE,       yyvsp[0].tree); ;
     break;}
 case 17:
-#line 113 "parser.y"
+#line 128 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_NOT_EQUAL, yyvsp[0].tree); ;
     break;}
 case 18:
-#line 114 "parser.y"
+#line 129 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_LTE,       yyvsp[0].tree); ;
     break;}
 case 19:
-#line 115 "parser.y"
+#line 130 "parser.y"
 { yyval.tree = yyvsp[-1].tree; ;
     break;}
 case 20:
-#line 117 "parser.y"
+#line 132 "parser.y"
 {
 		yyval.tree = p_new (ExprTree);
 		yyval.tree->ref_count = 1;
@@ -840,60 +855,55 @@ case 20:
 	;
     break;}
 case 21:
-#line 124 "parser.y"
+#line 139 "parser.y"
 {
 		yyval.tree = yyvsp[0].tree;
 	;
     break;}
 case 22:
-#line 128 "parser.y"
+#line 143 "parser.y"
 {
-		CellRef a, b;
-
-		a = yyvsp[-2].tree->u.ref;
-		b = yyvsp[0].tree->u.ref;
-
 		yyval.tree = p_new (ExprTree);
 		yyval.tree->ref_count = 1;
 		yyval.tree->oper = OPER_CONSTANT;
 		yyval.tree->u.constant = v_new ();
 		yyval.tree->u.constant->type = VALUE_CELLRANGE;
-		yyval.tree->u.constant->v.cell_range.cell_a = a;
-		yyval.tree->u.constant->v.cell_range.cell_b = b;
+		yyval.tree->u.constant->v.cell_range.cell_a = yyvsp[-2].tree->u.ref;
+		yyval.tree->u.constant->v.cell_range.cell_b = yyvsp[0].tree->u.ref;
 
 		forget_tree (yyvsp[-2].tree);
 		forget_tree (yyvsp[0].tree);
 	;
     break;}
 case 23:
-#line 147 "parser.y"
+#line 157 "parser.y"
 {
 		yyval.tree = yyvsp[-3].tree;
 		yyval.tree->u.function.arg_list = yyvsp[-1].list;
 	;
     break;}
 case 24:
-#line 153 "parser.y"
+#line 163 "parser.y"
 {
 		yyval.tree = yyvsp[0].tree;
 	;
     break;}
 case 25:
-#line 157 "parser.y"
+#line 167 "parser.y"
 {
 	        yyval.tree = yyvsp[0].tree;
 		yyval.tree->u.ref.sheet = yyvsp[-2].sheetref;
 	;
     break;}
 case 26:
-#line 163 "parser.y"
+#line 173 "parser.y"
 {
 		yyval.list = g_list_prepend (NULL, yyvsp[0].tree);
 		alloc_glist (yyval.list);
         ;
     break;}
 case 27:
-#line 167 "parser.y"
+#line 177 "parser.y"
 {
 		forget_glist (yyvsp[0].list);
 		yyval.list = g_list_prepend (yyvsp[0].list, yyvsp[-2].tree);
@@ -901,7 +911,7 @@ case 27:
 	;
     break;}
 case 28:
-#line 172 "parser.y"
+#line 182 "parser.y"
 { yyval.list = NULL; ;
     break;}
 }
@@ -1102,7 +1112,7 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 175 "parser.y"
+#line 185 "parser.y"
 
 
 static int
@@ -1146,7 +1156,7 @@ return_cellref (char *p)
 	/*  Ok, parsed successfully, create the return value */
 	e = p_new (ExprTree);
 	e->ref_count = 1;
-	
+
 	e->oper = OPER_VAR;
 
 	ref = &e->u.ref;
@@ -1165,7 +1175,7 @@ return_cellref (char *p)
 	ref->col_relative = col_relative;
 	ref->row_relative = row_relative;
 	ref->sheet = parser_sheet;
-	
+
 	yylval.tree = e;
 
 	return CELLREF;
@@ -1200,8 +1210,8 @@ make_string_return (char *string)
 	if (format_match (string, &fv, &format)){
 		v->type = VALUE_FLOAT;
 		v->v.v_float = fv;
-		if (!parser_desired_format)
-			parser_desired_format = format;
+		if (parser_desired_format && *parser_desired_format == NULL)
+			*parser_desired_format = format;
 	} else {
 		v->v.str = string_get (string);
 		v->type = VALUE_STRING;
@@ -1293,11 +1303,11 @@ try_symbol (char *string, gboolean try_cellref)
 int yylex (void)
 {
 	int c;
-	char *p, *tmp;
+	const char *p, *tmp;
 	int is_float, digits;
-	
+
         while(isspace (*parser_expr))
-                parser_expr++;                                                                                       
+                parser_expr++;
 
 	c = *parser_expr++;
         if (c == '(' || c == ',' || c == ')')
@@ -1384,7 +1394,7 @@ int yylex (void)
 	}
 	
 	if (isalpha (c) || c == '_' || c == '$'){
-		char *start = parser_expr - 1;
+		const char *start = parser_expr - 1;
 		char *str;
 		int  len;
 		
@@ -1397,10 +1407,8 @@ int yylex (void)
 		str [len] = 0;
 		return try_symbol (str, TRUE);
 	}
-	if (c == '\n')
-		return 0;
-	
-	if (c == EOF)
+
+	if (c == '\n' || c == 0)
 		return 0;
 
 	if (c == '<'){
@@ -1429,12 +1437,14 @@ int yylex (void)
 int
 yyerror (char *s)
 {
-    printf ("Error: %s\n", s);
-    return 0;
+#if 0
+	printf ("Error: %s\n", s);
+#endif
+	return 0;
 }
 
 static void
-alloc_register (void *a_info)
+alloc_register (AllocRec *a_info)
 {
 	alloc_list = g_list_prepend (alloc_list, a_info);
 }
@@ -1453,7 +1463,7 @@ void *
 alloc_buffer (int size)
 {
 	AllocRec *a_info = g_new (AllocRec, 1);
-	char *res = g_malloc (size);
+	void *res = g_malloc (size);
 
 	a_info->type = ALLOC_BUFFER;
 	a_info->data = res;
@@ -1462,11 +1472,11 @@ alloc_buffer (int size)
 	return res;
 }
 
-static void *
+static Value *
 v_new (void)
 {
 	AllocRec *a_info = g_new (AllocRec, 1);
-	char *res = g_malloc (sizeof (Value));
+	Value *res = g_new (Value, 1);
 
 	a_info->type = ALLOC_VALUE;
 	a_info->data = res;
@@ -1503,7 +1513,7 @@ alloc_clean (void)
 		g_free (rec);
 	}
 
-	g_list_free (l);
+	g_list_free (alloc_list);
 	alloc_list = NULL;
 }
 
@@ -1515,7 +1525,7 @@ alloc_list_free (void)
 	for (; l; l = l->next)
 		g_free (l->data);
 
-	g_list_free (l);
+	g_list_free (alloc_list);
 	alloc_list = NULL;
 }
 
@@ -1539,6 +1549,8 @@ forget (AllocType type, void *data)
 
 		if (a_info->type == type && a_info->data == data){
 			alloc_list = g_list_remove_link (alloc_list, l);
+			g_list_free_1 (l);
+			g_free (a_info);
 			return;
 		}
 	}
@@ -1553,12 +1565,12 @@ forget_glist (GList *list)
 static void
 forget_tree (ExprTree *tree)
 {
+	expr_tree_unref (tree);
 	forget (ALLOC_BUFFER, tree);
-	expr_tree_unref (tree); 
 }
 
 void
-value_dump (Value *value)
+value_dump (const Value *value)
 {
 	switch (value->type){
 	case VALUE_STRING:
@@ -1577,14 +1589,14 @@ value_dump (Value *value)
 		int x, y;
 		
 		printf ("Array: { ");
-		for (x = 0; x < value->v.array.x; x++)
-			for (y = 0; y < value->v.array.y; y++)
-				value_dump (&value->v.array.vals [x][y]);
+		for (y = 0; y < value->v.array.y; y++)
+			for (x = 0; x < value->v.array.x; x++)
+				value_dump (value->v.array.vals [x][y]);
 		printf ("}\n");
 		break;
 	}
 	case VALUE_CELLRANGE: {
-		CellRef *c = &value->v.cell_range.cell_a;
+		const CellRef *c = &value->v.cell_range.cell_a;
 		printf ("CellRange\n");
 		printf ("%p: %d,%d rel? %d,%d\n", c->sheet, c->col, c->row,
 			c->col_relative, c->row_relative);
@@ -1606,7 +1618,7 @@ dump_tree (ExprTree *tree)
 	
 	switch (tree->oper){
 	case OPER_VAR:
-		cr = &tree->u.constant->v.cell;
+		cr = &tree->u.ref;
 		printf ("Cell: %s%c%s%d\n",
 			cr->col_relative ? "" : "$",
 			cr->col + 'A',
@@ -1623,18 +1635,7 @@ dump_tree (ExprTree *tree)
 		printf ("Function call: %s\n", s->str);
 		break;
 
-	case OPER_EQUAL:
-	case OPER_NOT_EQUAL:
-	case OPER_LT:
-	case OPER_LTE:
-	case OPER_GT:
-	case OPER_GTE:
-	case OPER_ADD:
-	case OPER_SUB:
-	case OPER_MULT:
-	case OPER_DIV:
-	case OPER_EXP:
-	case OPER_CONCAT:
+	case OPER_ANY_BINARY:
 		dump_tree (tree->u.binary.value_a);
 		dump_tree (tree->u.binary.value_b);
 		switch (tree->oper){
@@ -1662,3 +1663,30 @@ dump_tree (ExprTree *tree)
 		
 	}
 }
+
+ParseErr
+gnumeric_expr_parser (const char *expr, Sheet *sheet, int col, int row,
+		      const char **desired_format, ExprTree **result)
+{
+	parser_error = PARSE_OK;
+	parser_expr = expr;
+	parser_sheet = sheet;
+	parser_col   = col;
+	parser_row   = row;
+	parser_desired_format = desired_format;
+	parser_result = result;
+
+	if (parser_desired_format)
+		*parser_desired_format = NULL;
+
+	yyparse ();
+
+	if (parser_error == PARSE_OK)
+		alloc_list_free ();
+	else {
+		alloc_clean ();
+		*parser_result = NULL;
+	}
+
+	return parser_error;
+}
