@@ -73,7 +73,7 @@ void
 sheet_object_container_land (SheetObject *so)
 {
 	SheetObjectContainer *soc;
-	GnomeObject *component;
+	GnomeObjectClient *component;
 	GList *l;
 
 	g_return_if_fail (so != NULL);
@@ -260,7 +260,7 @@ sheet_object_container_new (Sheet *sheet,
 {
 	SheetObjectContainer *c;
 	SheetObject *so;
-	GnomeObject *object_server;
+	GnomeObjectClient *object_server;
 	
 	g_return_val_if_fail (sheet != NULL, NULL);
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
