@@ -14,6 +14,7 @@
 #include "gnumeric.h"
 #include "gnumeric-sheet.h"
 #include "item-cursor.h"
+#include "utils.h"
 
 static GtkTableClass *sheet_view_parent_class;
 
@@ -689,7 +690,7 @@ sheet_view_comment_relocate (SheetView *sheet_view, int col, int row, GnomeCanva
 
 #ifdef ENABLE_BONOBO
 void
-sheet_view_insert_object (SheetView *sheet_view, GnomeObject *object)
+sheet_view_insert_object (SheetView *sheet_view, GnomeObjectClient *object)
 {
 	GtkWidget *view;
 	
