@@ -1,11 +1,19 @@
 #ifndef GNUMERIC_SHEET_H
 #define GNUMERIC_SHEET_H
 
+typedef struct _Workbook Workbook;
+typedef struct _Sheet Sheet;
+
 #ifdef ENABLE_BONOBO
 #    include <bonobo/gnome-container.h>
 #endif
 
 #include "solver.h"
+#include "style.h"
+#include "expr.h"
+#include "str.h"
+#include "symbol.h"
+#include "cell.h"
 
 #define SHEET_MAX_ROWS (16 * 1024)
 #define SHEET_MAX_COLS 256
@@ -24,7 +32,7 @@ typedef struct {
 	Style  *style;
 } StyleRegion;
 
-typedef struct {
+struct _Workbook {
 	char       *filename;
 
         GtkWidget  *toplevel; 
@@ -81,7 +89,8 @@ typedef struct {
 	GnomeContainer *gnome_container;
 	
 #endif
-} Workbook;
+	void       *corba_server;
+};
 
 typedef struct {
 	int        base_col, base_row;
@@ -106,7 +115,7 @@ typedef enum {
 	SHEET_MODE_OBJECT_SELECTED,
 } SheetModeType;
 
-typedef struct {
+struct _Sheet {
 	int         signature;
 	
 	Workbook    *workbook;
@@ -159,7 +168,9 @@ typedef struct {
 
         /* Solver parameters */
         SolverParameters solver_parameters;
-} Sheet;
+
+	void         *corba_server;
+};
 
 #define SHEET_SIGNATURE 0x12349876
 #define IS_SHEET(x) (((Sheet *) x)->signature == SHEET_SIGNATURE)
@@ -207,6 +218,7 @@ int         sheet_selection_first_range  (Sheet *sheet,
 					  int *start_col, int *start_row,
 					  int *end_col,   int *end_row);
 CellList   *sheet_selection_to_list      (Sheet *sheet);
+char       *sheet_selection_to_string    (Sheet *sheet, gboolean include_sheet_name_prefix);
 
 /* Operations on the selection */
 void        sheet_selection_clear             (Sheet *sheet);
@@ -227,7 +239,6 @@ int         sheet_selection_walk_step         (Sheet *sheet,
 void        sheet_selection_extend_horizontal (Sheet *sheet, int count);
 void        sheet_selection_extend_vertical   (Sheet *sheet, int count);
 int         sheet_selection_is_cell_selected  (Sheet *sheet, int col, int row);
-
 gboolean    sheet_verify_selection_simple     (Sheet *sheet, const char *command_name);
 
 /* Cell management */
@@ -242,7 +253,9 @@ int         sheet_cell_foreach_range      (Sheet *sheet, int only_existing,
 				           int end_col, int end_row,
 				           sheet_cell_foreach_callback callback,
 				           void *closure);
+ /* Returns NULL if doesn't exist */
 Cell       *sheet_cell_get                (Sheet *sheet, int col, int row);
+ /* Returns new Cell if doesn't exist */
 Cell       *sheet_cell_fetch              (Sheet *sheet, int col, int row);
 void        sheet_cell_comment_link       (Cell *cell);
 void        sheet_cell_comment_unlink     (Cell *cell);
@@ -322,7 +335,7 @@ void        sheet_redraw_all              (Sheet *sheet);
 void        sheet_update_auto_expr        (Sheet *sheet);
 
 void        sheet_mark_clean              (Sheet *sheet);
-
+void        sheet_set_dirty               (Sheet *sheet, gboolean is_dirty);
 /* Sheet information manipulation */
 void        sheet_insert_col              (Sheet *sheet,  int col, int count);
 void        sheet_delete_col              (Sheet *sheet,  int col, int count);
@@ -395,8 +408,9 @@ Workbook   *workbook_read                (const char *filename);
 void        workbook_save_as             (Workbook *);
 void        workbook_save                (Workbook *);
 void        workbook_attach_sheet        (Workbook *, Sheet *);
-gboolean    workbook_detach_sheet        (Workbook *, Sheet *);
+gboolean    workbook_detach_sheet        (Workbook *, Sheet *, gboolean);
 Sheet      *workbook_focus_current_sheet (Workbook *wb);
+void        workbook_focus_sheet         (Sheet *sheet);
 Sheet      *workbook_get_current_sheet   (Workbook *wb);
 char       *workbook_sheet_get_free_name (Workbook *wb);
 void        workbook_auto_expr_label_set (Workbook *wb, const char *text);
@@ -405,11 +419,14 @@ void        workbook_set_region_status   (Workbook *wb, const char *str);
 int         workbook_parse_and_jump      (Workbook *wb, const char *text);
 Sheet      *workbook_sheet_lookup        (Workbook *wb, const char *sheet_name);
 void        workbook_mark_clean          (Workbook *wb);
+void        workbook_set_dirty           (Workbook *wb, gboolean is_dirty);
 gboolean    workbook_rename_sheet        (Workbook *wb,
 					  const char *old_name,
 					  const char *new_name);
 int         workbook_sheet_count         (Workbook *wb);
 gboolean    workbook_can_detach_sheet    (Workbook *wb, Sheet *sheet);
+GList      *workbook_sheets              (Workbook *wb);
+char       *workbook_selection_to_string (Workbook *wb, Sheet *base_sheet);
 
 /*
  * Does any pending recalculations
@@ -428,6 +445,14 @@ typedef gboolean (*WorkbookCallback)(Workbook *, gpointer data);
 void     workbook_foreach             (WorkbookCallback cback,
 				       gpointer data);
 
+
+
+void	workbook_fixup_references	(Workbook *wb, Sheet *sheet, int col, int row,
+					 int coldelta, int rowdelta);
+
+void	workbook_invalidate_references	(Workbook *wb, Sheet *sheet, int col, int row,
+					 int colcount, int rowcount);
+
 /*
  * Feedback routines
  */
@@ -441,5 +466,14 @@ void     workbook_feedback_set        (Workbook *,
 				       void *data);
 
 extern   Workbook *current_workbook;
+
+/*
+ * Hooks for CORBA bootstrap: they create the 
+ */
+void workbook_corba_setup    (Workbook *);
+void workbook_corba_shutdown (Workbook *);
+
+void sheet_corba_setup       (Sheet *);
+void sheet_corba_shutdown    (Sheet *);
 #endif /* GNUMERIC_SHEET_H */
 
