@@ -167,6 +167,11 @@ parse_cell_name (const char *cell_str, int *col, int *row)
 		return FALSE;
 
 	(*row)--;
+
+	if (*col >= SHEET_MAX_COLS)
+		return FALSE;
+	if (*row >= SHEET_MAX_ROWS)
+		return FALSE;
 	
 	return TRUE;
 }
@@ -197,13 +202,13 @@ gnumeric_strcase_hash (gconstpointer v)
 }
 
 static guint32
-january_1900()
+january_1900 (void)
 {
 	static guint32 julian = 0;
 
 	if (!julian) {
 		GDate* date = g_date_new_dmy (1, 1, 1900);
-		/* Day 1 means 1st of jannuary of 1900 */
+		/* Day 1 means 1st of January of 1900 */
 		julian = g_date_julian (date) - 1;
 		g_date_free (date);
 	}
@@ -223,40 +228,51 @@ g_date_new_serial (guint32 serial)
 	return g_date_new_julian (serial + january_1900 ());
 }
 
-/* Returns a list of cells in a string.  If the named cells does not
+/*
+ * Returns a list of cells in a string.  If the named cells does not
  * exist, they are created.  If the input string is not valid
  * error_flag is set.
  */
-GSList  *parse_cell_name_list  (void *vsheet, 
-				const char *cell_name_str, int *error_flag)
+GSList *
+parse_cell_name_list (Sheet *sheet, 
+		      const char *cell_name_str,
+		      int *error_flag)
 {
-        Sheet    *sheet = (Sheet *) vsheet;
-        char     *buf = (char *) malloc(strlen(cell_name_str)+1);
+        char     *buf;
 	GSList   *cells = NULL;
 	Cell     *cell;
 	int      i, n, col, row;
 
-	for (i=n=0; 1; i++) {
-	        if ((cell_name_str[i] == ',') ||
-		    (cell_name_str[i] == '\0')) {
-		        buf[n] = '\0';
-			if (! parse_cell_name(buf, &col, &row)) {
+	g_return_val_if_fail (sheet != NULL, NULL);
+	g_return_val_if_fail (IS_SHEET (sheet), NULL);
+	g_return_val_if_fail (cell_name_str != NULL, NULL);
+	g_return_val_if_fail (error_flag != NULL, NULL);
+	
+	buf = g_malloc (strlen (cell_name_str) + 1);
+	for (i = n = 0; 1; i++){
+
+	        if ((cell_name_str [i] == ',') || (cell_name_str [i] == '\0')){
+		        buf [n] = '\0';
+
+			if (!parse_cell_name (buf, &col, &row)){
 			        *error_flag = 1;
 				free (buf);
 				g_slist_free (cells);
 				return NULL;
 			}
+
 			cell = sheet_cell_get (sheet, col, row);
-			if (cell == NULL) {
+
+			if (cell == NULL){
 			        cell = sheet_cell_new (sheet, col, row);
 				cell_set_text (cell, "");
 			}
 			cells = g_slist_append (cells, (gpointer) cell);
 			n = 0;
 		} else
-		        buf[n++] = cell_name_str[i];
+		        buf [n++] = cell_name_str [i];
 
-		if (cell_name_str[i] == '\0')
+		if (cell_name_str [i] == '\0')
 		        break;
 	}
 
@@ -264,3 +280,6 @@ GSList  *parse_cell_name_list  (void *vsheet,
 	free (buf);
 	return cells;
 }
+
+
+
