@@ -19,6 +19,8 @@
 #include "xml-io.h"
 #include "plugin.h"
 #include "pixmaps.h"
+#include "clipboard.h"
+#include "utils.h"
 #include "widget-editable-label.h"
 
 /* The locations within the main table in the workbook */
@@ -228,33 +230,72 @@ create_ellipse_cmd (GtkWidget *widget, Workbook *wb)
 }
 
 static void
-cb_sheet_do_destroy (gpointer key, gpointer value, gpointer user_data)
+cb_sheet_do_erase (gpointer key, gpointer value, gpointer user_data)
 {
 	Sheet *sheet = value;
 
-	sheet_destroy (sheet);
+	sheet_clear_region (sheet, 0, 0, SHEET_MAX_COLS - 1, SHEET_MAX_ROWS - 1);
 }
 
 static void
 workbook_do_destroy (Workbook *wb)
 {
-	
-	g_hash_table_foreach (wb->sheets, cb_sheet_do_destroy, NULL);
+	/* First do all deletions that leave the workbook in a working
+	   order.  */
+
+	/* Erase all cells.  In particular this removes all links between
+	   sheets.  */
+	g_hash_table_foreach (wb->sheets, cb_sheet_do_erase, NULL);
+
+	if (wb->auto_expr){
+		expr_tree_unref (wb->auto_expr);
+		wb->auto_expr = NULL;
+		string_unref (wb->auto_expr_desc);
+	}
+
+	/* Problems with insert/delete column/row caused formula_cell_list
+	   to be messed up.  */
+	if (wb->formula_cell_list) {
+		fprintf (stderr, "Reminder: FIXME in workbook_do_destroy\n");
+		g_list_free (wb->formula_cell_list);
+		wb->formula_cell_list = NULL;
+	}
+
+	/* Just drop the eval queue.  */
+	g_list_free (wb->eval_queue);
+	wb->eval_queue = NULL;
+
+	/* Detach and destroy all sheets.  */
+	{
+		GList *sheets, *l;
+
+		sheets = workbook_sheets (wb);
+		for (l = sheets; l ; l = l->next) {
+			Sheet *sheet = l->data;
+
+			workbook_detach_sheet (sheet->workbook, sheet, TRUE);
+			sheet_destroy (sheet);
+		}
+		g_list_free (sheets);
+	}
+
+	if (wb->clipboard_contents)
+		clipboard_release (wb->clipboard_contents);
+	wb->clipboard_contents = NULL;
+
+	/* Remove ourselves from the list of workbooks.  */
+	workbook_list = g_list_remove (workbook_list, wb);
+	workbook_count--;
+
+	/* Now do deletions that will put this workbook into a weird
+	   state.  Careful here.  */
+
 	g_hash_table_destroy (wb->sheets);
 
 	if (wb->filename)
 	       g_free (wb->filename);
 
-	g_list_free (wb->eval_queue);
-
 	g_free (wb->toolbar);
-	if (wb->auto_expr){
-		expr_tree_unref (wb->auto_expr);
-		string_unref (wb->auto_expr_desc);
-	}
-
-	workbook_list = g_list_remove (workbook_list, wb);
-	workbook_count--;
 
 	symbol_table_destroy (wb->symbol_names);
 
@@ -277,11 +318,20 @@ workbook_widget_destroy (GtkWidget *widget, Workbook *wb)
 }
 
 static void
-cb_sheet_mark_clean (gpointer key, gpointer value, gpointer user_data)
+cb_sheet_mark_dirty (gpointer key, gpointer value, gpointer user_data)
 {
 	Sheet *sheet = value;
+	int dirty = GPOINTER_TO_INT (user_data);
 
-	sheet_mark_clean (sheet);
+	sheet_set_dirty (sheet, dirty);
+}
+
+void
+workbook_set_dirty (Workbook *wb, gboolean is_dirty)
+{
+	g_return_if_fail (wb != NULL);
+
+	g_hash_table_foreach (wb->sheets, cb_sheet_mark_dirty, GINT_TO_POINTER (is_dirty));
 }
 
 void
@@ -289,7 +339,7 @@ workbook_mark_clean (Workbook *wb)
 {
 	g_return_if_fail (wb != NULL);
 
-	g_hash_table_foreach (wb->sheets, cb_sheet_mark_clean, NULL);
+	g_hash_table_foreach (wb->sheets, cb_sheet_mark_dirty, GINT_TO_POINTER (0));
 }
 
 typedef enum {
@@ -513,7 +563,7 @@ insert_cols_cmd (GtkWidget *unused, Workbook *wb)
 	int cols;
 	
 	sheet = workbook_get_current_sheet (wb);
-	if (!sheet_verify_selection_simple (sheet, _("Insert rows")))
+	if (!sheet_verify_selection_simple (sheet, _("Insert cols")))
 		return;
 	
 	ss = sheet->selections->data;
@@ -869,8 +919,6 @@ static GnomeUIInfo workbook_menu_format [] = {
 	{ GNOME_APP_UI_ITEM, N_("_Cells..."),
 	  N_("Modify the formatting of the selected cells"),
 	  format_cells_cmd, NULL, NULL, 0, 0, GDK_1, GDK_CONTROL_MASK },
-	{ GNOME_APP_UI_ITEM, N_("_Sort"),
-	  N_("Sort the selected cells"), sort_cells_cmd },
 #if 0
 	{ GNOME_APP_UI_SUBTREE, N_("C_olumn"), NULL, workbook_menu_format_column },
 	{ GNOME_APP_UI_SUBTREE, N_("_Row"),    NULL, workbook_menu_format_row },
@@ -881,6 +929,8 @@ static GnomeUIInfo workbook_menu_format [] = {
 
 /* Tools menu */
 static GnomeUIInfo workbook_menu_tools [] = {
+	{ GNOME_APP_UI_ITEM, N_("_Sort"),
+	  N_("Sort the selected cells"), sort_cells_cmd },
 	{ GNOME_APP_UI_ITEM, N_("_Goal Seek..."), NULL, goal_seek_cmd },
 #if 0
 	{ GNOME_APP_UI_ITEM, N_("_Solver..."),    NULL, solver_cmd },
@@ -901,7 +951,7 @@ static GnomeUIInfo workbook_menu [] = {
 	GNOMEUIINFO_MENU_VIEW_TREE(workbook_menu_view),
 	{ GNOME_APP_UI_SUBTREE, N_("_Insert"), NULL, workbook_menu_insert },
 	{ GNOME_APP_UI_SUBTREE, N_("F_ormat"), NULL, workbook_menu_format },
-	{ GNOME_APP_UI_SUBTREE, N_("T_ools"), NULL, workbook_menu_tools },
+	{ GNOME_APP_UI_SUBTREE, N_("_Tools"), NULL, workbook_menu_tools },
 	GNOMEUIINFO_MENU_HELP_TREE(workbook_menu_help),
 	GNOMEUIINFO_END
 };
@@ -1029,6 +1079,39 @@ workbook_focus_current_sheet (Workbook *wb)
 	return sheet;
 }
 
+void
+workbook_focus_sheet (Sheet *sheet)
+{
+	SheetView *sheet_view;
+	GtkNotebook *notebook;
+	int sheets, i;
+
+	g_return_if_fail (sheet);
+	g_return_if_fail (sheet->workbook);
+	g_return_if_fail (sheet->workbook->notebook);
+	g_return_if_fail (IS_SHEET (sheet));
+
+	notebook = GTK_NOTEBOOK (sheet->workbook->notebook);
+	sheets = workbook_sheet_count (sheet->workbook);
+
+	if (sheets == 1)
+		return;
+
+	for (i = 0; i < sheets; i++){
+		Sheet *this_sheet;
+		GtkWidget *w;
+
+		w = gtk_notebook_get_nth_page (notebook, i);
+
+		this_sheet = gtk_object_get_data (GTK_OBJECT (w), "sheet");
+
+		if (this_sheet == sheet){
+			gtk_notebook_set_page (notebook, i);
+			break;
+		}
+	}
+}
+
 static void
 wb_input_finished (GtkEntry *entry, Workbook *wb)
 {
@@ -1139,17 +1222,56 @@ wizard_input (GtkWidget *widget, Workbook *wb)
 	g_free (edittxt);
 }
 
+static void
+deps_output (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet = workbook_get_current_sheet (wb);
+	int col, row, dummy;
+	Cell *cell;
+	GList *list;
+
+	if (!sheet_selection_first_range (
+		sheet, &dummy, &dummy, &col, &row, &dummy, &dummy)){
+		gnumeric_notice (
+			wb, GNOME_MESSAGE_BOX_ERROR,
+			_("Selection must be a single range"));
+		return;
+	}
+	printf ("The cells that depend on %s\n", cell_name (col, row));
+
+	if (!(cell = sheet_cell_get (sheet, col, row))){
+		printf ("must contain some data\n");
+		return;
+	}
+
+	list = cell_get_dependencies (sheet, col, row);
+	if (!list)
+		printf ("No dependencies\n");
+
+	while (list){
+		Cell *cell = list->data;
+
+		list = g_list_next (list);
+		if (!cell)
+			continue;
+
+		if (sheet != cell->sheet && cell->sheet)
+			printf ("%s", cell->sheet->name);
+		
+		printf ("%s\n", cell_name (cell->col->pos, cell->row->pos));
+	}
+}
+
 static void
 workbook_setup_edit_area (Workbook *wb)
 {
-	GtkWidget *ok_button, *cancel_button, *wizard_button, *box, *box2;
-	GtkWidget *pix;
+	GtkWidget *ok_button, *cancel_button, *wizard_button;
+	GtkWidget *pix, *deps_button, *box, *box2;
 	
 	wb->ea_status = gtk_entry_new ();
 	wb->ea_input  = gtk_entry_new ();
 	ok_button     = gtk_button_new ();
 	cancel_button = gtk_button_new ();
-	wizard_button = gtk_button_new ();
 	box           = gtk_hbox_new (0, 0);
 	box2          = gtk_hbox_new (0, 0);
 	
@@ -1169,19 +1291,33 @@ workbook_setup_edit_area (Workbook *wb)
 	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
 			    GTK_SIGNAL_FUNC(cancel_input), wb);
 	
+
+	gtk_box_pack_start (GTK_BOX (box2), wb->ea_status, 0, 0, 0);
+	gtk_box_pack_start (GTK_BOX (box), ok_button, 0, 0, 0);
+	gtk_box_pack_start (GTK_BOX (box), cancel_button, 0, 0, 0);
+
 	/* Function Wizard, currently only enabled if you run with --debug=1 */
 	if (gnumeric_debugging){
+		wizard_button = gtk_button_new ();
 		pix = gnome_stock_pixmap_widget_new (wb->toplevel, GNOME_STOCK_PIXMAP_BOOK_GREEN);
 		gtk_container_add (GTK_CONTAINER (wizard_button), pix);
 		GTK_WIDGET_UNSET_FLAGS (wizard_button, GTK_CAN_FOCUS);
 		gtk_signal_connect (GTK_OBJECT (wizard_button), "clicked",
 				    GTK_SIGNAL_FUNC(wizard_input), wb);
+		gtk_box_pack_start (GTK_BOX (box), wizard_button, 0, 0, 0);
+	}
+	
+	/* Dependency Debugger, currently only enabled if you run with --debug=10 */
+	if (gnumeric_debugging > 9){
+		deps_button = gtk_button_new ();
+		pix = gnome_stock_pixmap_widget_new (wb->toplevel, GNOME_STOCK_PIXMAP_BOOK_RED);
+		gtk_container_add (GTK_CONTAINER (deps_button), pix);
+		GTK_WIDGET_UNSET_FLAGS (deps_button, GTK_CAN_FOCUS);
+		gtk_signal_connect (GTK_OBJECT (deps_button), "clicked",
+				    GTK_SIGNAL_FUNC(deps_output), wb);
+		gtk_box_pack_start (GTK_BOX (box), deps_button, 0, 0, 0);
 	}
 	
-	gtk_box_pack_start (GTK_BOX (box2), wb->ea_status, 0, 0, 0);
-	gtk_box_pack_start (GTK_BOX (box), ok_button, 0, 0, 0);
-	gtk_box_pack_start (GTK_BOX (box), cancel_button, 0, 0, 0);
-	gtk_box_pack_start (GTK_BOX (box), wizard_button, 0, 0, 0);
 	gtk_box_pack_start (GTK_BOX (box2), box, 0, 0, 0);
 	gtk_box_pack_end   (GTK_BOX (box2), wb->ea_input, 1, 1, 0);
 
@@ -1469,6 +1605,9 @@ workbook_new (void)
 #ifdef ENABLE_BONOBO
 	wb->gnome_container = GNOME_CONTAINER (gnome_container_new ());
 #endif
+
+	workbook_corba_setup (wb);
+	
 	return wb;
 }
 
@@ -1618,7 +1757,7 @@ sheet_action_delete_sheet (GtkWidget *widget, Sheet *current_sheet)
 	/*
 	 * All is fine, remove the sheet
 	 */
-	workbook_detach_sheet (wb, current_sheet);
+	workbook_detach_sheet (wb, current_sheet, FALSE);
 	sheet_destroy (current_sheet);
 	workbook_recalc_all (wb);
 }
@@ -1800,7 +1939,7 @@ workbook_can_detach_sheet (Workbook *wb, Sheet *sheet)
  * Detaches @sheet from the workbook @wb.
  */
 gboolean
-workbook_detach_sheet (Workbook *wb, Sheet *sheet)
+workbook_detach_sheet (Workbook *wb, Sheet *sheet, gboolean force)
 {
 	GtkNotebook *notebook;
 	int sheets, i;
@@ -1815,11 +1954,13 @@ workbook_detach_sheet (Workbook *wb, Sheet *sheet)
 	notebook = GTK_NOTEBOOK (wb->notebook);
 	sheets = workbook_sheet_count (sheet->workbook);
 
-	if (sheets == 1)
-		return FALSE;
+	if (!force) {
+		if (sheets == 1)
+			return FALSE;
 
-	if (!workbook_can_detach_sheet (wb, sheet))
-		return FALSE;
+		if (!workbook_can_detach_sheet (wb, sheet))
+			return FALSE;
+	}
 
 	/*
 	 * Remove our reference to this sheet
@@ -1986,7 +2127,7 @@ workbook_set_title (Workbook *wb, const char *title)
 	g_return_if_fail (wb != NULL);
 	g_return_if_fail (title != NULL);
 
-	full_title = g_strconcat ("Gnumeric: ", title, NULL);
+	full_title = g_strconcat (_("Gnumeric: "), title, NULL);
 	
  	gtk_window_set_title (GTK_WINDOW (wb->toplevel), full_title);
 	g_free (full_title);
@@ -1994,7 +2135,7 @@ workbook_set_title (Workbook *wb, const char *title)
 
 /**
  * workbook_set_filename:
- * @wb: the workkbook to modify
+ * @wb: the workbook to modify
  * @name: the file name for this worksheet.
  *
  * Sets the internal filename to @name and changes
@@ -2027,3 +2168,183 @@ workbook_foreach (WorkbookCallback cback, gpointer data)
 			return;
 	}
 }
+
+/**
+ * workbook_fixup_references:
+ * @wb: the workbook to modify
+ * @sheet: the sheet containing the column/row that was moved
+ * @col: starting column that was moved.
+ * @row: starting row that was moved.
+ * @coldelta: signed column distance that cells were moved.
+ * @rowcount: signed row distance that cells were moved.
+ *
+ * Fixes references after a column or row move.
+ */
+
+void
+workbook_fixup_references (Workbook *wb, Sheet *sheet, int col, int row,
+			   int coldelta, int rowdelta)
+{
+	GList *cells, *l;
+
+	g_return_if_fail (wb != NULL);
+
+	if (coldelta == 0 && rowdelta == 0) return;
+
+	/* Copy the list since it will change underneath us.  */
+	cells = g_list_copy (wb->formula_cell_list);
+
+	for (l = cells; l; l = l->next)	{
+		Cell *cell = l->data;
+		ExprTree *newtree;
+		int thiscol, thisrow;
+
+		thiscol = cell->col->pos;
+		thisrow = cell->row->pos;
+
+		newtree = expr_tree_fixup_references (cell->parsed_node, cell->sheet,
+						      thiscol, thisrow, sheet,
+						      col, row, coldelta, rowdelta);
+		if (newtree) {
+			char *exprtxt, *eqexprtxt;
+
+			exprtxt = expr_decode_tree (newtree, cell->sheet, thiscol, thisrow);
+			eqexprtxt = g_strconcat ("=", exprtxt, NULL);
+			cell_set_text (cell, eqexprtxt);
+
+			g_free (exprtxt);
+			g_free (eqexprtxt);
+			expr_tree_unref (newtree);
+		}
+	}
+
+	g_list_free (cells);
+}
+
+/**
+ * workbook_invalidate_references:
+ * @wb: the workbook to modify
+ * @sheet:  the sheet containing column and row to be invalidated
+ * @col: starting column to be invalidated.
+ * @row: starting row to be invalidated.
+ * @colcount: number of columns to be invalidated.
+ * @rowcount: number of rows to be invalidated.
+ *
+ * Invalidates *either* a number of columns *or* a number of rows.
+ */
+
+void
+workbook_invalidate_references (Workbook *wb, Sheet *sheet, int col, int row,
+				int colcount, int rowcount)
+{
+	GList *cells, *l;
+
+	g_return_if_fail (wb != NULL);
+	g_return_if_fail (colcount == 0 || rowcount == 0);
+	g_return_if_fail (colcount >= 0 && rowcount >= 0);
+
+	if (colcount == 0 && rowcount == 0) return;
+
+	/* Copy the list since it will change underneath us.  */
+	cells = g_list_copy (wb->formula_cell_list);
+
+	for (l = cells; l; l = l->next)	{
+		Cell *cell = l->data;
+		ExprTree *newtree;
+		int thiscol, thisrow;
+
+		thiscol = cell->col->pos;
+		thisrow = cell->row->pos;
+
+		newtree = expr_tree_invalidate_references (cell->parsed_node, cell->sheet,
+							   thiscol, thisrow, sheet,
+							   col, row, colcount, rowcount);
+		if (newtree) {
+			char *exprtxt, *eqexprtxt;
+
+			exprtxt = expr_decode_tree (newtree, cell->sheet, thiscol, thisrow);
+			eqexprtxt = g_strconcat ("=", exprtxt, NULL);
+			cell_set_text (cell, eqexprtxt);
+
+			g_free (exprtxt);
+			g_free (eqexprtxt);
+			expr_tree_unref (newtree);
+		}
+	}
+
+	g_list_free (cells);
+}
+
+
+
+static void
+cb_workbook_sheets (gpointer key, gpointer value, gpointer user_data)
+{
+	Sheet *sheet = value;
+	GList **l = user_data;
+
+	*l = g_list_prepend (*l, sheet);
+}
+
+
+GList *
+workbook_sheets (Workbook *wb)
+{
+	GList *sheets = NULL;
+
+	g_return_val_if_fail (wb != NULL, NULL);
+	g_hash_table_foreach (wb->sheets, cb_workbook_sheets, &sheets);
+	return sheets;
+}
+
+typedef struct {
+	Sheet   *base_sheet;
+	GString *result;
+} selection_assemble_closure_t;
+
+static void
+cb_assemble_selection (gpointer key, gpointer value, gpointer user_data)
+{
+	selection_assemble_closure_t *info = (selection_assemble_closure_t *) user_data;
+	Sheet *sheet = value;
+	gboolean include_prefix;
+	char *sel;
+	
+	if (*info->result->str)
+		g_string_append_c (info->result, ',');
+
+	/*
+	 * If a base sheet is specified, use this to avoid prepending
+	 * the full path to the cell region.
+	 */
+	if (info->base_sheet && (info->base_sheet != value))
+		include_prefix = TRUE;
+	else
+		include_prefix = FALSE;
+	
+	sel = sheet_selection_to_string (sheet, include_prefix);
+	g_string_append (info->result, sel);
+	g_free (sel);
+}
+
+char *
+workbook_selection_to_string (Workbook *wb, Sheet *base_sheet)
+{
+	selection_assemble_closure_t info;
+	char *str, *result;
+	
+	g_return_val_if_fail (wb != NULL, NULL);
+
+	if (base_sheet == NULL){
+		g_return_val_if_fail (IS_SHEET (base_sheet), NULL);
+	}
+
+	info.result = g_string_new ("");
+	info.base_sheet = base_sheet;
+	g_hash_table_foreach (wb->sheets, cb_assemble_selection, &info);
+
+	result = info.result->str;
+	g_string_free (info.result, FALSE);
+
+	return result;
+}
