@@ -3,7 +3,7 @@
  *
  * Daniel Veillard <Daniel.Veillard@w3.org>
  *
- * $Id: xml-io.c,v 1.41 1999/04/18 02:52:30 unammx Exp $
+ * $Id: xml-io.c,v 1.44 1999/05/17 23:27:02 unammx Exp $
  */
 
 #include <config.h>
@@ -56,7 +56,7 @@ xmlGetValue (xmlNodePtr node, const char *name)
 
 	ret = (char *) xmlGetProp (node, name);
 	if (ret != NULL)
-		return strdup (ret);
+		return ret;
 	child = node->childs;
 
 	while (child != NULL) {
@@ -268,7 +268,7 @@ xmlSetGnomeCanvasPoints (xmlNodePtr node, const char *name,
 		return;
 	if ((val->num_points < 0) || (val->num_points > 5000))
 		return;
-	base = str = malloc (val->num_points * 30 * sizeof (char));
+	base = str = g_malloc (val->num_points * 30 * sizeof (char));
 	if (str == NULL)
 		return;
 	for (i = 0; i < val->num_points; i++){
@@ -286,7 +286,7 @@ xmlSetGnomeCanvasPoints (xmlNodePtr node, const char *name,
 		child = child->next;
 	}
 	xmlNewChild (node, NULL, name, xmlEncodeEntities(node->doc, base));
-	free (base);
+	g_free (base);
 }
 
 
@@ -770,18 +770,16 @@ writeXmlStyleBorder (parseXmlContextPtr ctxt, StyleBorder *border)
 	for (lp = 3; lp >= 0; lp--)
 		if (border->type [lp] != BORDER_NONE)
 			break;
-	if (lp<0)
+	if (lp < 0)
 		return NULL;
 
 	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "StyleBorder", NULL);
 	
-	for (lp=0;lp<4;lp++)
- 	{
- 		if (border->type[lp] != BORDER_NONE)
- 		{
- 			side = xmlNewChild (cur, ctxt->ns, StyleSideNames[lp],
- 					    BorderTypes[border->type[lp]]);
- 			xmlSetColorValue (side, "Color", border->color[lp]);
+	for (lp = 0; lp < 4; lp++){
+ 		if (border->type[lp] != BORDER_NONE){
+ 			side = xmlNewChild (cur, ctxt->ns, StyleSideNames [lp],
+ 					    BorderTypes [border->type [lp]]);
+ 			xmlSetColorValue (side, "Color", border->color [lp]);
  		}
 	}
 	return cur;
@@ -886,7 +884,7 @@ static Style *
 readXmlStyle (parseXmlContextPtr ctxt, xmlNodePtr tree, Style * ret)
 {
 	xmlNodePtr child;
-	const char *prop;
+	char *prop;
 	int val;
 	StyleColor *c;
 
@@ -932,15 +930,16 @@ readXmlStyle (parseXmlContextPtr ctxt, xmlNodePtr tree, Style * ret)
 	prop = xmlGetValue (tree, "Format");
 	if (prop != NULL){
 		if (ret->format == NULL){
-			ret->format = style_format_new ((char *) prop);
+			ret->format = style_format_new ((const char *) prop);
 			ret->valid_flags |= STYLE_FORMAT;
 		}
+		free (prop);
 	}
 
 	child = tree->childs;
 	while (child != NULL){
 		if (!strcmp (child->name, "Font")){
-			const char *v;
+			char *v;
 
 			v = xmlGetValue (child, "NAME");
 			if (v){
@@ -957,6 +956,7 @@ readXmlStyle (parseXmlContextPtr ctxt, xmlNodePtr tree, Style * ret)
 					g_hash_table_insert (ctxt->nameTable, g_strdup (v), ret->font);
 					ret->valid_flags |= STYLE_FONT;
 				}
+				free (v);
 			} else {
 				StyleFont *font;
 
@@ -1243,6 +1243,8 @@ readXmlCell (parseXmlContextPtr ctxt, xmlNodePtr tree)
 	xmlGetIntValue (tree, "Col", &col);
 	xmlGetIntValue (tree, "Row", &row);
 
+	cell_deep_freeze_redraws ();
+
 	ret = sheet_cell_get (ctxt->sheet, col, row);
 	if (ret == NULL)
 		ret = sheet_cell_new (ctxt->sheet, col, row);
@@ -1280,6 +1282,8 @@ readXmlCell (parseXmlContextPtr ctxt, xmlNodePtr tree)
 	} else
 		cell_set_text_simple (ret, "");
 
+	cell_deep_thaw_redraws ();
+
 	return ret;
 }
 
@@ -1398,7 +1402,7 @@ readXmlSheet (parseXmlContextPtr ctxt, xmlNodePtr tree)
 	xmlNodePtr cells;
 	xmlNodePtr objects;
 	Sheet *ret = NULL;
-	const char *val;
+	char *val;
 
 	if (strcmp (tree->name, "Sheet")){
 		fprintf (stderr,
@@ -1417,6 +1421,7 @@ readXmlSheet (parseXmlContextPtr ctxt, xmlNodePtr tree)
 		ret = workbook_sheet_lookup (ctxt->wb, (const char *) val);
 		if (ret == NULL)
 			ret = sheet_new (ctxt->wb, (const char *) val);
+		free (val);
 	} 
 
 	if (ret == NULL)
@@ -1535,7 +1540,7 @@ writeXmlWorkbook (parseXmlContextPtr ctxt, Workbook *wb)
 static void
 createXmlSheet (parseXmlContextPtr ctxt, xmlNodePtr tree)
 {
-	const char *val;
+	char *val;
 	xmlNodePtr child;
 	
 	if (strcmp (tree->name, "Sheet")){
@@ -1549,8 +1554,9 @@ createXmlSheet (parseXmlContextPtr ctxt, xmlNodePtr tree)
 	if (val != NULL){
 		Sheet *sheet;
 		
-		sheet = sheet_new (ctxt->wb, (char *) val);
+		sheet = sheet_new (ctxt->wb, (const char *) val);
 		workbook_attach_sheet (ctxt->wb, sheet);
+		free (val);
 	}
 	return;
 }
@@ -1622,13 +1628,15 @@ xml_probe (const char *filename)
 {
 	xmlDocPtr res;
 	xmlNsPtr gmr;
-	
+
 	res = xmlParseFile (filename);
 	if (res == NULL)
 		return FALSE;
 
-	if (res->root == NULL)
+	if (res->root == NULL) {
+		xmlFreeDoc (res);
 		return FALSE;
+	}
 
 	gmr = xmlSearchNsByHref (res, res->root, "http://www.gnome.org/gnumeric/");
 	if (res->root->name == NULL || strcmp (res->root->name, "Workbook") || (gmr == NULL)){
